package simpledb;

import java.util.*;

/**
 * The Catalog keeps track of all available tables in the database and their
 * associated schemas.
 * For now, this is a stub catalog that must be populated with tables by a
 * user program before it can be used -- eventually, this should be converted
 * to a catalog that reads a catalog table from disk.
 */

public class Catalog {

    // <silentstrip lab1>
    HashMap<Integer, DbFile> id2table;
    HashMap<Integer, TupleDesc> id2tupledesc;
    HashMap<String, Integer> name2id;
    // </silentstrip>

    /**
     * Constructor.
     * Creates a new, empty catalog.
     */
    public Catalog() {
        // <strip lab1>
        id2table = new HashMap<Integer, DbFile>();
        id2tupledesc = new HashMap<Integer, TupleDesc>();
        name2id = new HashMap<String,Integer>();
        // </strip>
    }

    /**
     * Add a new table to the catalog.
     * This table has tuples formatted using the specified TupleDesc and its
     * contents are stored in the specified DbFile.
     * @param file the contents of the table to add;  file.id() is the identfier of
     *    this file/tupledesc param for the calls getTupleDesc and getFile
     * @param t the format of tuples that are being added
     * @param name the name of the table -- may be an empty string.  May not be null.  If a name
     * conflict exists, use the last table to be added as the table for a given name.
     */
    public void addTable(DbFile file, TupleDesc t, String name) {
        // <strip lab1>
        id2tupledesc.put(new Integer(file.id()), t);
        id2table.put(new Integer(file.id()), file);
        name2id.put(name, new Integer(file.id()));
        // </strip>
    }

    /**
     * Add a new table to the catalog.
     * This table has tuples formatted using the specified TupleDesc and its
     * contents are stored in the specified DbFile.
     * @param file the contents of the table to add;  file.id() is the identfier of
     *    this file/tupledesc param for the calls getTupleDesc and getFile
     * @param t the format of tuples that are being added
     */
    public void addTable(DbFile file, TupleDesc t) {
        addTable(file, t, "");
    }


    /**
     * Return the id of the table with a specified name,
     * @throws NoSuchElementException if the table doesn't exist
     */
    public int getTableId(String name) {
        // <strip lab1>
        if (name2id.get(name) == null)
            throw new NoSuchElementException();

        return name2id.get(name).intValue();
        // </strip>
        // <insert lab1>
        // return 0;
        // </insert>
    }

    /**
     * Returns the tuple descriptor (schema) of the specified table
     * @param tableid The id of the table, as specified by the DbFile.id()
     *     function passed to addTable
     */
    public TupleDesc getTupleDesc(int tableid) throws NoSuchElementException {
        // <strip lab1>
        return id2tupledesc.get(new Integer(tableid));
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * Returns the DbFile that can be used to read the contents of the
     * specified table.
     * @param tableid The id of the table, as specified by the DbFile.id()
     *     function passed to addTable
     */
    public DbFile getDbFile(int tableid) throws NoSuchElementException {
        // <strip lab1>
        return id2table.get(new Integer(tableid));
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }


    /** Delete all tables from the catalog */
    public void clear() {
        // <strip lab1>
        id2table.clear();
        id2tupledesc.clear();
        // </strip>
    }
}
