package simpledb;
import java.util.*;

/**
 * SeqScan is an implementation of a sequential scan access method that reads
 * each tuple of a table in no particular order (e.g., as they are laid out on
 * disk).
 */
public class SeqScan implements DbIterator {
    // <silentstrip lab1>
    boolean isOpen = false;
    TransactionId tid;
    TupleDesc myTd;
    DbFileIterator it;
    // </silentstrip>

    /**
     * Creates a sequential scan over the specified table as a part of the
     * specified transaction.
     *
     * @param tid The transaction this scan is running as a part of.
     * @param tableid the table to scan.
     * @param tableAlias the alias of this table (needed by the parser);
     *         the returned tupleDesc should have fields with name tableAlias.fieldName
     *         (note: this class is not responsible for handling a case where tableAlias
     *         or fieldName are null.  It shouldn't crash if they are, but the resulting
     *         name can be null.fieldName, tableAlias.null, or null.null).
     */
    public SeqScan(TransactionId tid, int tableid, String tableAlias) {
        // <strip lab1>
        this.tid = tid;
        this.it = Database.getCatalog().getDbFile(tableid).iterator(tid);
        myTd = Database.getCatalog().getTupleDesc(tableid);
        String[] newNames = new String[myTd.numFields()];
        Type[] newTypes = new Type[myTd.numFields()];
        for (int i = 0; i < myTd.numFields(); i++) {
            String name = myTd.getFieldName(i);
            Type t = myTd.getType(i);

            newNames[i] = tableAlias + "." + name;
            newTypes[i] = t;
        }
        myTd = new TupleDesc(newTypes, newNames);
        // </strip>
    }

    public void open()
        throws DbException, TransactionAbortedException {
        // <strip lab1>
        if (isOpen)
            throw new DbException("double open on one DbIterator.");

        it.open();
        isOpen = true;
        // </strip>
    }

    /**
     * Returns the TupleDesc with field names from the underlying HeapFile,
     * prefixed with the tableAlias string from the constructor.
     * @return the TupleDesc with field names from the underlying HeapFile,
     * prefixed with the tableAlias string from the constructor.
     */
    public TupleDesc getTupleDesc() {
        // <strip lab1>
        return myTd;
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    public boolean hasNext() throws TransactionAbortedException, DbException {
        // <strip lab1>
        if (!isOpen)
            throw new IllegalStateException("iterator is closed");
        return it.hasNext();
        // </strip>
        // <insert lab1>
        // return false;
        // </insert>
    }

    public Tuple next()
        throws NoSuchElementException, TransactionAbortedException, DbException {
        // <strip lab1>
        if (!isOpen)
            throw new IllegalStateException("iterator is closed");

        return it.next();

        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    public void close() {
        // <strip lab1>
        it.close();
        isOpen = false;
        // </strip>
    }

    public void rewind()
        throws DbException, NoSuchElementException, TransactionAbortedException {
        // <strip lab1>
        close();
        open();
        // </strip>
    }
}
