package simpledb;

/**
 * Tuple maintains information about the contents of a tuple.
 * Tuples have a specified schema specified by a TupleDesc object and contain
 * Field objects with the data for each field.
 */
public class Tuple {
    // <silentstrip lab1>
    RecordID rid; //source on disk -- may be null
    Field fields[];
    TupleDesc td;
    // </silentstrip>

    /**
     * Create a new tuple with the specified schema (type).
     *
     * @param td the schema of this tuple. It must be a valid TupleDesc
     * instance with at least one field.
     */
    public Tuple(TupleDesc td) {
        // <strip lab1>
        fields = new Field[td.numFields()];
        this.td = td;
        // </strip>
    }

    /**
     * @return The TupleDesc representing the schema of this tuple.
     */
    public TupleDesc getTupleDesc() {
        // <strip lab1>
        return td;
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * @return The RecordID representing the location of this tuple on
     *   disk. May be null.
     */
    public RecordID getRecordID() {
        // <strip lab1>
        return rid;
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * Set the RecordID information for this tuple.
     * @param rid the new RecordID for this tuple.
     */
    public void setRecordID(RecordID rid) {
        // <strip lab1>
        this.rid = rid;
        // </strip>
    }

    /**
     * Change the value of the ith field of this tuple.
     *
     * @param i index of the field to change. It must be a valid index.
     * @param f new value for the field.
     */
    public void setField(int i, Field f) {
        // <strip lab1>
        fields[i] = f;
        // </strip>
    }

    /**
     * @return the value of the ith field, or null if it has not been set.
     *
     * @param i field index to return. Must be a valid index.
     */
    public Field getField(int i) {
        // <strip lab1>
        return fields[i];
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * Returns the contents of this Tuple as a string.
     * Note that to pass the system tests, the format needs to be as
     * follows:
     *
     * column1\tcolumn2\tcolumn3\t...\tcolumnN\n
     *
     * where \t is any whitespace, except newline, and \n is a newline
     */
    public String toString() {
        // <strip lab1>
        String out = "";
        for (int i = 0; i<  fields.length; i++) {
            if (out.length() > 0 ) out += "\t";
            out += fields[i];
        }
        return out;
        // </strip>
        // <insert lab1>
        // throw new UnsupportedOperationException("Implement this");
        // </insert>
    }
}
