package simpledb;
import java.util.*;

/**
 * TupleDesc describes the schema of a tuple.
 */
public class TupleDesc {

    // <silentstrip lab1>
    Type[] tupleAr = null;
    String[] fields = null;

    private TupleDesc() {
    }
    // </silentstrip>

    /**
     * Merge two TupleDescs into one, with td1.numFields + td2.numFields
     * fields, with the first td1.numFields coming from td1 and the remaining
     * from td2.
     * @param td1 The TupleDesc with the first fields of the new TupleDesc
     * @param td2 The TupleDesc with the last fields of the TupleDesc
     * @return the new TupleDesc
     */
    public static TupleDesc combine(TupleDesc td1, TupleDesc td2) {
        // <strip lab1>
        TupleDesc comb = new TupleDesc();
        comb.tupleAr = new Type[td1.numFields() + td2.numFields()];
        comb.fields = new String[td1.numFields() + td2.numFields()];

        for(int i=0; i<td1.numFields(); i++) {
            comb.tupleAr[i] = td1.getType(i);
            comb.fields[i] = td1.getFieldName(i);
        }
        for(int i=0; i<td2.numFields(); i++) {
            comb.tupleAr[td1.numFields() + i] = td2.getType(i);
            comb.fields[td1.numFields() + i] = td2.getFieldName(i);

        }
        return comb;
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * Create a new TupleDesc with typeAr.length fields with fields of the
     * specified types, with associated named fields.
     *
     * @param typeAr array specifying the number of and types of fields in
     *        this TupleDesc. It must contain at least one entry.
     * @param fieldAr array specifying the names of the fields. Note that names may be null.
     */
    public TupleDesc(Type[] typeAr, String[] fieldAr) {
        // <strip lab1>
        tupleAr = (Type[]) typeAr.clone();
        fields = (String[]) fieldAr.clone();
        // </strip>
    }

    /**
     * Constructor.
     * Create a new tuple desc with typeAr.length fields with fields of the
     * specified types, with anonymous (unnamed) fields.
     *
     * @param typeAr array specifying the number of and types of fields in
     *        this TupleDesc. It must contain at least one entry.
     */
    public TupleDesc(Type[] typeAr) {
        // <strip lab1>
        tupleAr = (Type[]) typeAr.clone();
    fields = new String[tupleAr.length];
    for (int i = 0; i < fields.length; i++) {
        fields[i] = "";
    }
        // </strip>
    }

    /**
     * @return the number of fields in this TupleDesc
     */
    public int numFields() {
        // <strip lab1>
        return tupleAr.length;
        // </strip>
        // <insert lab1>
        // return 0;
        // </insert>
    }

    /**
     * Gets the (possibly null) field name of the ith field of this TupleDesc.
     *
     * @param i index of the field name to return. It must be a valid index.
     * @return the name of the ith field
     * @throws NoSuchElementException if i is not a valid field reference.
     */
    public String getFieldName(int i) throws NoSuchElementException {
        //<strip lab1>
        return fields[i];
        //</strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * Find the index of the field with a given name.
     *
     * @param name name of the field.
     * @return the index of the field that is first to have the given name.
     * @throws NoSuchElementException if no field with a matching name is found.
     */
    public int nameToId(String name) throws NoSuchElementException {
        //<strip lab1>
        for (int i = 0 ; i < numFields() ; i ++) {
            if (fields[i] != null && fields[i].equals(name))
            return i;
        }
        throw new NoSuchElementException("No field named " + name + " found");
        //</strip>
        // <insert lab1>
        // return 0;
        // </insert>
    }

    /**
     * Gets the type of the ith field of this TupleDesc.
     *
     * @param i The index of the field to get the type of. It must be a valid index.
     * @return the type of the ith field
     * @throws NoSuchElementException if i is not a valid field reference.
     */
    public Type getType(int i) throws NoSuchElementException {
        // <strip lab1>
        return tupleAr[i];
        // </strip>
        // <insert lab1>
        // return null;
        // </insert>
    }

    /**
     * @return The size (in bytes) of tuples corresponding to this TupleDesc.
     * Note that tuples from a given TupleDesc are of a fixed size.
     */
    public int getSize() {
        // <strip lab1>
        int size = 0;
        for(int i=0; i < tupleAr.length; i++)
            size += tupleAr[i].getLen();
        return size;
        // </strip>
        // <insert lab1>
        // return 0;
        // </insert>
    }

    /**
     * Compares the specified object with this TupleDesc for equality.
     * Two TupleDescs are considered equal if they are the same size and if the
     * n-th type in this TupleDesc is equal to the n-th type in td.
     *
     * @param o the Object to be compared for equality with this TupleDesc.
     * @return true if the object is equal to this TupleDesc.
     */
    public boolean equals(Object o) {
        // <strip lab1>
        if (!(o instanceof TupleDesc)) return false;
        TupleDesc td = (TupleDesc) o;

        if (tupleAr.length != td.tupleAr.length) return false;
        for (int i = 0; i < tupleAr.length; i++) {
            if (!tupleAr[i].equals(td.tupleAr[i]))
                return false;
        }
        return true;
        // </strip>
        // <insert lab1>
        // return false;
        // </insert>
    }

    public int hashCode() {
        // If you want to use TupleDesc as keys for HashMap, implement this so
        // that equal objects have equals hashCode() results
        throw new UnsupportedOperationException("unimplemented");
    }

    /**
     * Returns a String describing this descriptor. It should be of the form
     * "fieldType[0](fieldName[0]), ..., fieldType[M](fieldName[M])", although
     * the exact format does not matter.
     * @return String describing this descriptor.
     */
    public String toString() {
        // <strip lab1>
        String s= "";
        for (int i = 0; i < tupleAr.length; i++) {
            s += tupleAr[i] + "(" + fields[i] + "), ";
        }
        return s;
        // </strip>
        // <insert lab1>
        // return "";
        // </insert>
    }
}
